`include "internal_defines.vh"

//// branch_unit: Handles branch calculations and next-instruction selection
////
//// nextpc        (output) - Address of next instruction to execute
//// pc            (input)  - Address of currently executing instruction
//// bu__absimm    (input)  - Immediate form of absolute jump
//// bu__absalu    (input)  - ALU-result form of absolute jump
//// bu__offset    (input)  - Offset for relative jump
//// bu__select    (input)  - What sort of branch to compute
//// bu__condition (input)  - Whether the condition for this branch is true
////
module branch_unit(/*AUTOARG*/
   // Outputs
   nextpc,
   // Inputs
   pc, bu__absimm, bu__absalu, bu__offset, bu__select, bu__condition
   );

   output reg [31:0] nextpc;
   input [31:0]  pc;
   input [25:0]  bu__absimm;
   input [31:0]  bu__absalu;
   input [31:0]  bu__offset;
   input [1:0] 	 bu__select;
   input 	 bu__condition;

   always @(*) begin
      nextpc = pc;

      case (bu__select)
	`BU_NEXT:
	  nextpc = pc + 4;
	// FIXME: This disagrees with the spec but is necessary to pass brtest0
	`BU_CONDREL:
	  nextpc = pc + (bu__condition ? bu__offset + 4 : 4);
	`BU_ABSIMM:
	  nextpc = {pc[31:28], bu__absimm, 2'b0 };
	`BU_ABSALU:
	  nextpc = bu__absalu;
      endcase // case (bu__select)
   end
endmodule // branch_unit
