`include "mips_defines.vh"
`include "internal_defines.vh"

////
//// mips_alu: Performs all arithmetic and logical operations
////
//// alu__out   (output) - Final result
//// alu__op1   (input)  - Operand modified by the operation
//// alu__op2   (input)  - Operand used (in arithmetic ops) to modify in1
//// alu__shamt (input)  - Specifies shift amount for constant shift operations
//// aku__opsel (input)  - Selects which operation is to be performed
////
module mips_alu(/*AUTOARG*/
   // Outputs
   alu__out, oflow_halt,
   // Inputs
   alu__op1, alu__op2, alu__shamt, alu__opsel
   );

   output reg [31:0]    alu__out;
   output reg 		oflow_halt;
   input signed [31:0]  alu__op1, alu__op2;
   input [4:0] 	        alu__shamt;
   input [4:0] 	        alu__opsel;

   wire [31:0] 		alu__op1u, alu__op2u;
   
   assign alu__op1u = alu__op1;
   assign alu__op2u = alu__op2;
   
   always @(*) begin
      alu__out = 32'bx;
      oflow_halt = 1'b0;
      
      case (alu__opsel)
	`ALU_ADD: begin
	   alu__out = alu__op1 + alu__op2;
	   oflow_halt = (alu__op1[31] == alu__op2[31]) && (alu__out[31] != alu__op1[31]);
	end

	`ALU_ADDU:
	  alu__out = alu__op1 + alu__op2;
	
	`ALU_AND:
	  alu__out = alu__op1 & alu__op2;
			    
	`ALU_SUB: begin
	   alu__out = alu__op1 - alu__op2;
	   oflow_halt = (alu__op1[31] == ~alu__op2[31]) && (alu__out[31] != alu__op1[31]);
	end

	`ALU_SUBU:
	  alu__out = alu__op1 - alu__op2;
			    
	`ALU_OR:
	  alu__out = alu__op1 | alu__op2;

  	`ALU_NOR:
	  alu__out = ~(alu__op1 | alu__op2);

	`ALU_SLL:
	  alu__out = alu__op2 << alu__shamt;

  	`ALU_SLLV:
	  alu__out = alu__op2 << alu__op1[4:0];

	`ALU_XOR:
	  alu__out = alu__op1 ^ alu__op2;

	`ALU_SRA:
	  alu__out = alu__op2 >>> alu__shamt;

  	`ALU_SRAV:
	  alu__out = alu__op2 >>> alu__op1[4:0];

	`ALU_SRL:
	  alu__out = alu__op2 >> alu__shamt;

  	`ALU_SRLV:
	  alu__out = alu__op2 >> alu__op1[4:0];

	`ALU_LUI:
	  alu__out = {alu__op2[15:0], 16'b0};

	`ALU_OP1:
	  alu__out = alu__op1;

	`ALU_EQ:
	  alu__out = (alu__op1 == alu__op2) ? 32'b1 : 32'b0;

	`ALU_NE:
	  alu__out = (alu__op1 != alu__op2) ? 32'b1 : 32'b0;
	
	`ALU_LT:
	  alu__out = (alu__op1 < alu__op2) ? 32'b1 : 32'b0;
	
	`ALU_LE:
	  alu__out = (alu__op1 <= alu__op2) ? 32'b1 : 32'b0;

	`ALU_GT:
	  alu__out = (alu__op1 > alu__op2) ? 32'b1 : 32'b0;
	
	`ALU_GE:
	  alu__out = (alu__op1 >= alu__op2) ? 32'b1 : 32'b0;

	`ALU_LTU:
	  alu__out = (alu__op1u < alu__op2u) ? 32'b1 : 32'b0;
      endcase // case (alu__opsel)
   end
endmodule
