`include "mips_defines.vh"
`include "internal_defines.vh"
module mips_control(/*AUTOARG*/
   // Outputs
   alu__opsel, ctrl_RI, ctrl_Sys, ctrl_alurtsel, ctrl_immse,
   ctrl_rdnum, ctrl_rddata, reg__rd_we, reg__rs_re, reg__rt_re,
   mc__size, mc__se, mc__we, mul__active, mul__opcode,
   // Inputs
   dcd_op, dcd_funct2, dcd_rt
   );

   output reg [4:0]  alu__opsel;
   // output reg [1:0]  bu__select;
   output reg 	     ctrl_RI;
   output reg 	     ctrl_Sys;
   output reg 	     ctrl_alurtsel;
   output reg 	     ctrl_immse;
   output reg [1:0]  ctrl_rdnum;
   output reg [1:0]  ctrl_rddata;
   output reg 	     reg__rd_we;
   output reg 	     reg__rs_re;
   output reg 	     reg__rt_re;
   output reg [1:0]  mc__size;
   output reg 	     mc__se;
   output reg        mc__we;
   output reg 	     mul__active;
   output reg [2:0]  mul__opcode;
   input [5:0] 	     dcd_op;
   input [5:0] 	     dcd_funct2;
   input [4:0]       dcd_rt;

   always @(*) begin
      alu__opsel = 4'hx;
      // bu__select = `BU_NEXT;
      ctrl_Sys = 1'b0;
      ctrl_RI = 1'b0;
      ctrl_rdnum = `RDNUM_RT;
      ctrl_rddata = `RDDATA_ALU;
      ctrl_alurtsel = 1'b0;
      ctrl_immse = 1'b0;
      mc__we = 1'b0;
      mc__se = 1'b0;
      mc__size = `MC_DISABLED;
      mul__active = 1'b0;
      mul__opcode = 3'bx;
      reg__rd_we = 1'b0;
      reg__rs_re = 1'b0;
      reg__rt_re = 1'b0;

      case(dcd_op)
	`OP_OTHER0:
	  begin
	     ctrl_rdnum = `RDNUM_RD;
	     ctrl_alurtsel = 1'b1;
             case(dcd_funct2)
               `OP0_SYSCALL:
                 ctrl_Sys = 1'b1;

	       `OP0_ADD:
		 begin
		    alu__opsel = `ALU_ADD;
		    reg__rd_we = 1'b1;
		    reg__rs_re = 1'b1;
		    reg__rt_re = 1'b1;
		 end

	       `OP0_ADDU:
		 begin
		    alu__opsel = `ALU_ADDU;
		    reg__rd_we = 1'b1;
		    reg__rs_re = 1'b1;
		    reg__rt_re = 1'b1;
		 end

	       `OP0_AND:
		 begin
		    alu__opsel = `ALU_AND;
		    reg__rd_we = 1'b1;
		    reg__rs_re = 1'b1;
		    reg__rt_re = 1'b1;
		 end

	       `OP0_DIV:
		 begin
		    mul__active = 1'b1;
		    mul__opcode = `MUL_DIV;
		    reg__rs_re = 1'b1;
		    reg__rt_re = 1'b1;
		 end

	       `OP0_DIVU:
		 begin
		    mul__active = 1'b1;
		    mul__opcode = `MUL_DIVU;
		    reg__rs_re = 1'b1;
		    reg__rt_re = 1'b1;
		 end

	       // Disabled for Lab 2
	       //`OP0_JALR:
	       // begin
	       //    reg__rd_we = 1'b1;
	       //    ctrl_rddata = `RDDATA_PC;
	       //    alu__opsel = `ALU_OP1;
	       //    bu__select = `BU_ABSALU;
	       // end

               // Disabled for Lab 2
	       //`OP0_JR:
	       // begin
	       //    alu__opsel = `ALU_OP1;
	       //    bu__select = `BU_ABSALU;
	       // end

	       `OP0_MFHI:
		 begin
		    ctrl_rddata = `RDDATA_MULT;
		    mul__active = 1'b1;
		    mul__opcode = `MUL_MFHI;
		    reg__rd_we = 1'b1;
		 end

	       `OP0_MFLO:
		 begin
		    ctrl_rddata = `RDDATA_MULT;
		    mul__active = 1'b1;
		    mul__opcode = `MUL_MFLO;
		    reg__rd_we = 1'b1;
		 end

	       `OP0_MTHI:
		 begin
		    mul__active = 1'b1;
		    mul__opcode = `MUL_MTHI;
		    reg__rs_re = 1'b1;
		 end

	       `OP0_MTLO:
		 begin
		    mul__active = 1'b1;
		    mul__opcode = `MUL_MTLO;
		    reg__rs_re = 1'b1;
		 end

	       `OP0_MULT:
		 begin
		    mul__active = 1'b1;
		    mul__opcode = `MUL_MULT;
		    reg__rs_re = 1'b1;
		    reg__rt_re = 1'b1;
		 end

	       `OP0_MULTU:
		 begin
		    mul__active = 1'b1;
		    mul__opcode = `MUL_MULTU;
		    reg__rs_re = 1'b1;
		    reg__rt_re = 1'b1;
		 end

	       `OP0_NOR:
		 begin
		    alu__opsel = `ALU_NOR;
		    reg__rd_we = 1'b1;
		    reg__rs_re = 1'b1;
		    reg__rt_re = 1'b1;
		 end

	       `OP0_OR:
		 begin
		    alu__opsel = `ALU_OR;
		    reg__rd_we = 1'b1;
		    reg__rs_re = 1'b1;
		    reg__rt_re = 1'b1;
		 end

	       // Also, NOP
	       `OP0_SLL:
		 begin
		    alu__opsel = `ALU_SLL;
		    reg__rd_we = 1'b1;
		    reg__rt_re = 1'b1;
		 end

	       `OP0_SLLV:
		 begin
		    alu__opsel = `ALU_SLLV;
		    reg__rd_we = 1'b1;
		    reg__rs_re = 1'b1;
		    reg__rt_re = 1'b1;
		 end

	       `OP0_SLT:
		 begin
		    alu__opsel = `ALU_LT;
		    reg__rd_we = 1'b1;
		    reg__rs_re = 1'b1;
		    reg__rt_re = 1'b1;
		 end

	       `OP0_SLTU:
		 begin
		    alu__opsel = `ALU_LTU;
		    reg__rd_we = 1'b1;
		    reg__rs_re = 1'b1;
		    reg__rt_re = 1'b1;
		 end

	       `OP0_SRA:
		 begin
		    alu__opsel = `ALU_SRA;
		    reg__rd_we = 1'b1;
		    reg__rt_re = 1'b1;
		 end

	       `OP0_SRAV:
		 begin
		    alu__opsel = `ALU_SRAV;
		    reg__rd_we = 1'b1;
		    reg__rs_re = 1'b1;
		    reg__rt_re = 1'b1;
		 end

	       `OP0_SRL:
		 begin
		    alu__opsel = `ALU_SRL;
		    reg__rd_we = 1'b1;
		    reg__rt_re = 1'b1;
		 end

	       `OP0_SRLV:
		 begin
		    alu__opsel = `ALU_SRLV;
		    reg__rd_we = 1'b1;
		    reg__rs_re = 1'b1;
		    reg__rt_re = 1'b1;
		 end

	       `OP0_SUB:
		 begin
		    alu__opsel = `ALU_SUB;
		    reg__rd_we = 1'b1;
		    reg__rs_re = 1'b1;
		    reg__rt_re = 1'b1;
		 end

	       `OP0_SUBU:
		 begin
		    alu__opsel = `ALU_SUBU;
		    reg__rd_we = 1'b1;
		    reg__rs_re = 1'b1;
		    reg__rt_re = 1'b1;
		 end

	       `OP0_XOR:
		 begin
		    alu__opsel = `ALU_XOR;
		    reg__rd_we = 1'b1;
		    reg__rs_re = 1'b1;
		    reg__rt_re = 1'b1;
		 end

               default:
                 ctrl_RI = 1'b1;
             endcase // case (dcd_funct2)
	  end

	`OP_OTHER1:
	  begin
	     case (dcd_rt)
	       // Disabled for Lab 2
	       //`OP1_BLTZ:
	       // begin
	       //    ctrl_alurtsel = 1'b1;
	       //    alu__opsel = `ALU_LT;
	       //    bu__select = `BU_CONDREL;
	       // end

	       // Disabled for Lab 2
	       //`OP1_BLTZAL:
	       // begin
	       //    reg__rd_we = 1'b1;
	       //    ctrl_rdnum = `RDNUM_LR;
	       //    ctrl_rddata = `RDDATA_PC;
	       //    ctrl_alurtsel = 1'b1;
	       //    alu__opsel = `ALU_LT;
	       //    bu__select = `BU_CONDREL;
	       // end

	       // Disabled for Lab 2
	       //`OP1_BGEZ:
	       // begin
	       //    ctrl_alurtsel = 1'b1;
	       //    alu__opsel = `ALU_GE;
	       //    bu__select = `BU_CONDREL;
	       // end

	       // Disabled for Lab 2
	       //`OP1_BGEZAL:
	       // begin
	       //    reg__rd_we = 1'b1;
	       //    ctrl_rdnum = `RDNUM_LR;
	       //    ctrl_rddata = `RDDATA_PC;
	       //    ctrl_alurtsel = 1'b1;
	       //    alu__opsel = `ALU_GE;
	       //    bu__select = `BU_CONDREL;
	       // end

	       default:
		 ctrl_RI = 1'b1;
	     endcase // case (dcd_rt)
	  end

	`OP_ADDI:
	  begin
	     alu__opsel = `ALU_ADD;
	     reg__rd_we = 1'b1;
	     reg__rs_re = 1'b1;
	     ctrl_immse = 1'b1;
	  end

	`OP_ADDIU:
          begin
             alu__opsel = `ALU_ADDU;
             reg__rd_we = 1'b1;
	     reg__rs_re = 1'b1;
	     ctrl_immse = 1'b1;
          end

	`OP_ANDI:
	  begin
	     alu__opsel = `ALU_AND;
	     reg__rs_re = 1'b1;
	     reg__rd_we = 1'b1;
	  end

	// Disabled for Lab 2
	//`OP_BEQ:
	//  begin
	//     ctrl_alurtsel = 1'b1;
	//     alu__opsel = `ALU_EQ;
	//     bu__select = `BU_CONDREL;
	//  end

	// Disabled for Lab 2
	// `OP_BGTZ:
	//  begin
	//     ctrl_alurtsel = 1'b1;
	//     alu__opsel = `ALU_GT;
	//     bu__select = `BU_CONDREL;
	//  end

	// Disabled for Lab 2
	//`OP_BLEZ:
	//  begin
	//     ctrl_alurtsel = 1'b1;
	//     alu__opsel = `ALU_LE;
	//     bu__select = `BU_CONDREL;
	//  end

	// Disabled for Lab 2
	//`OP_BNE:
	//  begin
	//     ctrl_alurtsel = 1'b1;
	//     alu__opsel = `ALU_NE;
	//     bu__select = `BU_CONDREL;
	//  end

	// Disabled for Lab 2
	//`OP_J:
	//  begin
	//     bu__select = `BU_ABSIMM;
	//  end

	// Disabled for Lab 2
	//`OP_JAL:
	//  begin
	//     reg__rd_we = 1'b1;
	//     ctrl_rdnum = `RDNUM_LR;
	//     ctrl_rddata = `RDDATA_PC;
	//     bu__select = `BU_ABSIMM;
	//  end

	`OP_LB:
	  begin
	     alu__opsel = `ALU_ADDU;
	     ctrl_immse = 1'b1;
	     ctrl_rddata = `RDDATA_MC;
	     mc__se = 1'b1;
	     mc__size = `MC_BYTE;
	     reg__rd_we = 1'b1;
	     reg__rs_re = 1'b1;
	  end

	`OP_LBU:
	  begin
	     alu__opsel = `ALU_ADDU;
	     ctrl_immse = 1'b1;
	     ctrl_rddata = `RDDATA_MC;
	     mc__size = `MC_BYTE;
	     reg__rd_we = 1'b1;
	     reg__rs_re = 1'b1;
	  end

	`OP_LH:
	  begin
	     alu__opsel = `ALU_ADDU;
	     ctrl_immse = 1'b1;
	     ctrl_rddata = `RDDATA_MC;
	     mc__se = 1'b1;
	     mc__size = `MC_HALFWORD;
	     reg__rd_we = 1'b1;
	     reg__rs_re = 1'b1;
	  end

	`OP_LHU:
	  begin
	     alu__opsel = `ALU_ADDU;
	     ctrl_immse = 1'b1;
	     ctrl_rddata = `RDDATA_MC;
	     mc__size = `MC_HALFWORD;
	     reg__rd_we = 1'b1;
	     reg__rs_re = 1'b1;
	  end

	`OP_LUI:
	  begin
	     alu__opsel = `ALU_LUI;
	     reg__rd_we = 1'b1;
	  end

	`OP_LW:
	  begin
	     alu__opsel = `ALU_ADDU;
	     ctrl_immse = 1'b1;
	     ctrl_rddata = `RDDATA_MC;
	     mc__size = `MC_WORD;
	     reg__rd_we = 1'b1;
	     reg__rs_re = 1'b1;
	  end

	`OP_ORI:
	  begin
	     alu__opsel = `ALU_OR;
	     reg__rd_we = 1'b1;
	     reg__rs_re = 1'b1;
	  end

	`OP_SB:
	  begin
	     alu__opsel = `ALU_ADDU;
	     ctrl_immse = 1'b1;
	     mc__size = `MC_BYTE;
	     mc__we = 1'b1;
	     reg__rs_re = 1'b1;
	     reg__rt_re = 1'b1;
	  end

	`OP_SH:
	  begin
	     alu__opsel = `ALU_ADDU;
	     ctrl_immse = 1'b1;
	     mc__size = `MC_HALFWORD;
	     mc__we = 1'b1;
	     reg__rs_re = 1'b1;
	     reg__rt_re = 1'b1;
	  end

	`OP_SLTI:
	  begin
	     alu__opsel = `ALU_LT;
	     ctrl_immse = 1'b1;
	     reg__rd_we = 1'b1;
	     reg__rs_re = 1'b1;
	  end

	`OP_SLTIU:
	  begin
	     alu__opsel = `ALU_LTU;
	     ctrl_immse = 1'b1;
	     reg__rd_we = 1'b1;
	     reg__rs_re = 1'b1;
	  end

	`OP_SW:
	  begin
	     alu__opsel = `ALU_ADDU;
	     ctrl_immse = 1'b1;
	     mc__size = `MC_WORD;
	     mc__we = 1'b1;
	     reg__rs_re = 1'b1;
	     reg__rt_re = 1'b1;
	  end

	`OP_XORI:
	  begin
	     alu__opsel = `ALU_XOR;
	     reg__rd_we = 1'b1;
	     reg__rs_re = 1'b1;
	  end

	default:
          ctrl_RI = 1'b1;
      endcase // case(op)
   end
endmodule // mips_control
