/*
 *
 * Redistributions of any form whatsoever must retain and/or include the
 * following acknowledgment, notices and disclaimer:
 *
 * This product includes software developed by Carnegie Mellon University.
 *
 * Copyright (c) 2004 by Babak Falsafi and James Hoe,
 * Computer Architecture Lab at Carnegie Mellon (CALCM),
 * Carnegie Mellon University.
 *
 * This source file was written and maintained by Jared Smolens
 * as part of the Two-Way In-Order Superscalar project for Carnegie Mellon's
 * Introduction to Computer Architecture course, 18-447. The source file
 * is in part derived from code originally written by Herman Schmit and
 * Diana Marculescu.
 *
 * You may not use the name "Carnegie Mellon University" or derivations
 * thereof to endorse or promote products derived from this software.
 *
 * If you modify the software you must place a notice on or within any
 * modified version provided or made available to any third party stating
 * that you have modified the software.  The notice shall include at least
 * your name, address, phone number, email address and the date and purpose
 * of the modification.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" WITHOUT ANY WARRANTY OF ANY KIND, EITHER
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO ANYWARRANTY
 * THAT THE SOFTWARE WILL CONFORM TO SPECIFICATIONS OR BE ERROR-FREE AND ANY
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
 * TITLE, OR NON-INFRINGEMENT.  IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY
 * BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO DIRECT, INDIRECT,
 * SPECIAL OR CONSEQUENTIAL DAMAGES, ARISING OUT OF, RESULTING FROM, OR IN
 * ANY WAY CONNECTED WITH THIS SOFTWARE (WHETHER OR NOT BASED UPON WARRANTY,
 * CONTRACT, TORT OR OTHERWISE).
 *
 */

//////
////// MIPS: A MIPS ISA simulator
//////

// Include the MIPS constants
`include "mips_defines.vh"
`include "internal_defines.vh"

////
//// The MIPS standalone processor module
////
////   clk          (input)  - The clock
////   inst_addr    (output) - Address of instruction to load
////   inst         (input)  - Instruction from memory
////   inst_excpt   (input)  - inst_addr not valid
////   mem_addr     (output) - Address of data to load
////   mem_data_in  (output) - Data for memory store
////   mem_data_out (input)  - Data from memory load
////   mem_write_en (output) - Memory write mask
////   mem_excpt    (input)  - mem_addr not valid
////   halted       (output) - Processor halted
////   reset        (input)  - Reset the processor
////
module mips_core(/*AUTOARG*/
   // Outputs
   inst_addr, mem_addr, mem_data_in, mem_write_en, halted,
   // Inputs
   clk, inst_excpt, mem_excpt, inst, mem_data_out, rst_b
   );

   // Static interface
   input         clk, inst_excpt, mem_excpt;
   output [29:0] inst_addr;
   output [29:0] mem_addr;
   input  [31:0] inst, mem_data_out;
   output [31:0] mem_data_in;
   output [3:0]  mem_write_en;
   output reg    halted;
   input         rst_b;

   // Automatics
   /*AUTOWIRE*/
   // Beginning of automatic wires (for undeclared instantiated-module outputs)
   wire [4:0]		alu__opsel_2a;		// From ID of mips_core_id.v
   wire [31:0]		alu__out_3a;		// From EX of mips_core_ex.v
   wire			bubble_b_1a;		// From IF of mips_core_if.v
   wire			bubble_b_2a;		// From ID of mips_core_id.v
   wire			bubble_b_3a;		// From EX of mips_core_ex.v
   wire			bubble_b_4a;		// From MEM of mips_core_mem.v
   wire			ctrl_RI_2a;		// From ID of mips_core_id.v
   wire			ctrl_RI_3a;		// From EX of mips_core_ex.v
   wire			ctrl_RI_4a;		// From MEM of mips_core_mem.v
   wire			ctrl_Sys_2a;		// From ID of mips_core_id.v
   wire			ctrl_Sys_3a;		// From EX of mips_core_ex.v
   wire			ctrl_Sys_4a;		// From MEM of mips_core_mem.v
   wire			ctrl_alurtsel_2a;	// From ID of mips_core_id.v
   wire			ctrl_immse_2a;		// From ID of mips_core_id.v
   wire [1:0]		ctrl_rddata_2a;		// From ID of mips_core_id.v
   wire [1:0]		ctrl_rddata_3a;		// From EX of mips_core_ex.v
   wire			exception_halt;		// From WB of mips_core_wb.v
   wire [31:0]		inst_1a;		// From IF of mips_core_if.v
   wire [31:0]		inst_2a;		// From ID of mips_core_id.v
   wire			inst_excpt_1a;		// From IF of mips_core_if.v
   wire			inst_excpt_2a;		// From ID of mips_core_id.v
   wire			inst_excpt_3a;		// From EX of mips_core_ex.v
   wire			inst_excpt_4a;		// From MEM of mips_core_mem.v
   wire			mc__AdEL_data_4a;	// From MEM of mips_core_mem.v
   wire			mc__AdES_4a;		// From MEM of mips_core_mem.v
   wire			mc__DBE_4a;		// From MEM of mips_core_mem.v
   wire			mc__se_2a;		// From ID of mips_core_id.v
   wire			mc__se_3a;		// From EX of mips_core_ex.v
   wire [1:0]		mc__size_2a;		// From ID of mips_core_id.v
   wire [1:0]		mc__size_3a;		// From EX of mips_core_ex.v
   wire			mc__we_2a;		// From ID of mips_core_id.v
   wire			mc__we_3a;		// From EX of mips_core_ex.v
   wire			mul__active_2a;		// From ID of mips_core_id.v
   wire [2:0]		mul__opcode_2a;		// From ID of mips_core_id.v
   wire [31:0]		mul__rd_data_3a;	// From EX of mips_core_ex.v
   wire [31:0]		next_alu__out_2a;	// From EX of mips_core_ex.v
   wire [31:0]		next_reg__rd_data_3a;	// From MEM of mips_core_mem.v
   wire			oflow_halt_3a;		// From EX of mips_core_ex.v
   wire			oflow_halt_4a;		// From MEM of mips_core_mem.v
   wire [31:0]		pc_1a;			// From IF of mips_core_if.v
   wire [31:0]		pc_2a;			// From ID of mips_core_id.v
   wire [31:0]		pc_3a;			// From EX of mips_core_ex.v
   wire [31:0]		pc_4a;			// From MEM of mips_core_mem.v
   wire [31:0]		reg__rd_data_1a;	// From WB of mips_core_wb.v
   wire [31:0]		reg__rd_data_4a;	// From MEM of mips_core_mem.v
   wire [4:0]		reg__rd_num_1a;		// From WB of mips_core_wb.v
   wire [4:0]		reg__rd_num_2a;		// From ID of mips_core_id.v
   wire [4:0]		reg__rd_num_3a;		// From EX of mips_core_ex.v
   wire [4:0]		reg__rd_num_4a;		// From MEM of mips_core_mem.v
   wire			reg__rd_we_1a;		// From WB of mips_core_wb.v
   wire			reg__rd_we_2a;		// From ID of mips_core_id.v
   wire			reg__rd_we_3a;		// From EX of mips_core_ex.v
   wire			reg__rd_we_4a;		// From MEM of mips_core_mem.v
   wire [31:0]		reg__rs_data_2a;	// From ID of mips_core_id.v
   wire [31:0]		reg__rt_data_2a;	// From ID of mips_core_id.v
   wire [31:0]		reg__rt_data_3a;	// From EX of mips_core_ex.v
   wire			stall_1a;		// From ID of mips_core_id.v
   wire			stall_2a;		// From EX of mips_core_ex.v
   wire			stall_3a;		// From MEM of mips_core_mem.v
   wire			stall_4a;		// From WB of mips_core_wb.v
   wire			syscall_halt;		// From WB of mips_core_wb.v
   // End of automatics

   // Pipeline stages
   mips_core_if IF(.inst_0a(inst),
		   /*AUTOINST*/
		   // Outputs
		   .inst_addr		(inst_addr[29:0]),
		   .bubble_b_1a		(bubble_b_1a),
		   .inst_1a		(inst_1a[31:0]),
		   .pc_1a		(pc_1a[31:0]),
		   .inst_excpt_1a	(inst_excpt_1a),
		   // Inputs
		   .clk			(clk),
		   .rst_b		(rst_b),
		   .inst_excpt		(inst_excpt),
		   .stall_1a		(stall_1a),
		   .stall_2a		(stall_2a),
		   .stall_3a		(stall_3a),
		   .stall_4a		(stall_4a));
   
   mips_core_id ID(/*AUTOINST*/
		   // Outputs
		   .alu__opsel_2a	(alu__opsel_2a[4:0]),
		   .bubble_b_2a		(bubble_b_2a),
		   .ctrl_RI_2a		(ctrl_RI_2a),
		   .ctrl_Sys_2a		(ctrl_Sys_2a),
		   .ctrl_alurtsel_2a	(ctrl_alurtsel_2a),
		   .ctrl_immse_2a	(ctrl_immse_2a),
		   .ctrl_rddata_2a	(ctrl_rddata_2a[1:0]),
		   .inst_2a		(inst_2a[31:0]),
		   .inst_excpt_2a	(inst_excpt_2a),
		   .mc__size_2a		(mc__size_2a[1:0]),
		   .mc__se_2a		(mc__se_2a),
		   .mc__we_2a		(mc__we_2a),
		   .mul__active_2a	(mul__active_2a),
		   .mul__opcode_2a	(mul__opcode_2a[2:0]),
		   .pc_2a		(pc_2a[31:0]),
		   .reg__rd_num_2a	(reg__rd_num_2a[4:0]),
		   .reg__rd_we_2a	(reg__rd_we_2a),
		   .reg__rs_data_2a	(reg__rs_data_2a[31:0]),
		   .reg__rt_data_2a	(reg__rt_data_2a[31:0]),
		   .stall_1a		(stall_1a),
		   // Inputs
		   .clk			(clk),
		   .halted		(halted),
		   .rst_b		(rst_b),
		   .bubble_b_1a		(bubble_b_1a),
		   .inst_1a		(inst_1a[31:0]),
		   .inst_excpt_1a	(inst_excpt_1a),
		   .next_alu__out_2a	(next_alu__out_2a[31:0]),
		   .next_reg__rd_data_3a(next_reg__rd_data_3a[31:0]),
		   .pc_1a		(pc_1a[31:0]),
		   .reg__rd_data_1a	(reg__rd_data_1a[31:0]),
		   .reg__rd_num_1a	(reg__rd_num_1a[4:0]),
		   .reg__rd_we_1a	(reg__rd_we_1a),
		   .stall_2a		(stall_2a),
		   .stall_3a		(stall_3a),
		   .stall_4a		(stall_4a));
   
   mips_core_ex EX(/*AUTOINST*/
		   // Outputs
		   .alu__out_3a		(alu__out_3a[31:0]),
		   .bubble_b_3a		(bubble_b_3a),
		   .ctrl_RI_3a		(ctrl_RI_3a),
		   .ctrl_Sys_3a		(ctrl_Sys_3a),
		   .ctrl_rddata_3a	(ctrl_rddata_3a[1:0]),
		   .inst_excpt_3a	(inst_excpt_3a),
		   .mc__size_3a		(mc__size_3a[1:0]),
		   .mc__se_3a		(mc__se_3a),
		   .mc__we_3a		(mc__we_3a),
		   .mul__rd_data_3a	(mul__rd_data_3a[31:0]),
		   .next_alu__out_2a	(next_alu__out_2a[31:0]),
		   .oflow_halt_3a	(oflow_halt_3a),
		   .pc_3a		(pc_3a[31:0]),
		   .reg__rd_num_3a	(reg__rd_num_3a[4:0]),
		   .reg__rd_we_3a	(reg__rd_we_3a),
		   .reg__rt_data_3a	(reg__rt_data_3a[31:0]),
		   .stall_2a		(stall_2a),
		   // Inputs
		   .clk			(clk),
		   .rst_b		(rst_b),
		   .alu__opsel_2a	(alu__opsel_2a[4:0]),
		   .bubble_b_2a		(bubble_b_2a),
		   .ctrl_RI_2a		(ctrl_RI_2a),
		   .ctrl_Sys_2a		(ctrl_Sys_2a),
		   .ctrl_alurtsel_2a	(ctrl_alurtsel_2a),
		   .ctrl_immse_2a	(ctrl_immse_2a),
		   .ctrl_rddata_2a	(ctrl_rddata_2a[1:0]),
		   .inst_2a		(inst_2a[31:0]),
		   .inst_excpt_2a	(inst_excpt_2a),
		   .mc__size_2a		(mc__size_2a[1:0]),
		   .mc__se_2a		(mc__se_2a),
		   .mc__we_2a		(mc__we_2a),
		   .mul__active_2a	(mul__active_2a),
		   .mul__opcode_2a	(mul__opcode_2a[2:0]),
		   .pc_2a		(pc_2a[31:0]),
		   .reg__rd_num_2a	(reg__rd_num_2a[4:0]),
		   .reg__rd_we_2a	(reg__rd_we_2a),
		   .reg__rs_data_2a	(reg__rs_data_2a[31:0]),
		   .reg__rt_data_2a	(reg__rt_data_2a[31:0]),
		   .stall_3a		(stall_3a),
		   .stall_4a		(stall_4a));
    
   mips_core_mem MEM(/*AUTOINST*/
		     // Outputs
		     .mem_addr		(mem_addr[29:0]),
		     .mem_data_in	(mem_data_in[31:0]),
		     .mem_write_en	(mem_write_en[3:0]),
		     .bubble_b_4a	(bubble_b_4a),
		     .ctrl_RI_4a	(ctrl_RI_4a),
		     .ctrl_Sys_4a	(ctrl_Sys_4a),
		     .inst_excpt_4a	(inst_excpt_4a),
		     .mc__AdEL_data_4a	(mc__AdEL_data_4a),
		     .mc__AdES_4a	(mc__AdES_4a),
		     .mc__DBE_4a	(mc__DBE_4a),
		     .next_reg__rd_data_3a(next_reg__rd_data_3a[31:0]),
		     .oflow_halt_4a	(oflow_halt_4a),
		     .pc_4a		(pc_4a[31:0]),
		     .reg__rd_data_4a	(reg__rd_data_4a[31:0]),
		     .reg__rd_num_4a	(reg__rd_num_4a[4:0]),
		     .reg__rd_we_4a	(reg__rd_we_4a),
		     .stall_3a		(stall_3a),
		     // Inputs
		     .mem_data_out	(mem_data_out[31:0]),
		     .mem_excpt		(mem_excpt),
		     .clk		(clk),
		     .rst_b		(rst_b),
		     .alu__out_3a	(alu__out_3a[31:0]),
		     .bubble_b_3a	(bubble_b_3a),
		     .ctrl_RI_3a	(ctrl_RI_3a),
		     .ctrl_Sys_3a	(ctrl_Sys_3a),
		     .ctrl_rddata_3a	(ctrl_rddata_3a[1:0]),
		     .inst_excpt_3a	(inst_excpt_3a),
		     .mc__size_3a	(mc__size_3a[1:0]),
		     .mc__se_3a		(mc__se_3a),
		     .mc__we_3a		(mc__we_3a),
		     .mul__rd_data_3a	(mul__rd_data_3a[31:0]),
		     .oflow_halt_3a	(oflow_halt_3a),
		     .pc_3a		(pc_3a[31:0]),
		     .reg__rd_num_3a	(reg__rd_num_3a[4:0]),
		     .reg__rd_we_3a	(reg__rd_we_3a),
		     .reg__rt_data_3a	(reg__rt_data_3a[31:0]),
		     .stall_4a		(stall_4a));
   
   mips_core_wb WB(/*AUTOINST*/
		   // Outputs
		   .reg__rd_data_1a	(reg__rd_data_1a[31:0]),
		   .reg__rd_num_1a	(reg__rd_num_1a[4:0]),
		   .reg__rd_we_1a	(reg__rd_we_1a),
		   .stall_4a		(stall_4a),
		   .exception_halt	(exception_halt),
		   .syscall_halt	(syscall_halt),
		   // Inputs
		   .clk			(clk),
		   .rst_b		(rst_b),
		   .bubble_b_4a		(bubble_b_4a),
		   .ctrl_RI_4a		(ctrl_RI_4a),
		   .ctrl_Sys_4a		(ctrl_Sys_4a),
		   .inst_excpt_4a	(inst_excpt_4a),
		   .mc__AdEL_data_4a	(mc__AdEL_data_4a),
		   .mc__AdES_4a		(mc__AdES_4a),
		   .mc__DBE_4a		(mc__DBE_4a),
		   .oflow_halt_4a	(oflow_halt_4a),
		   .pc_4a		(pc_4a[31:0]),
		   .reg__rd_data_4a	(reg__rd_data_4a[31:0]),
		   .reg__rd_num_4a	(reg__rd_num_4a[4:0]),
		   .reg__rd_we_4a	(reg__rd_we_4a));

   // Debug spam
   // synthesis translate_off
   mips_debug DEBUG(/*AUTOINST*/
		    // Inputs
		    .clk		(clk),
		    .rst_b		(rst_b),
		    .halted		(halted),
		    .bubble_b_1a	(bubble_b_1a),
		    .bubble_b_2a	(bubble_b_2a),
		    .bubble_b_3a	(bubble_b_3a),
		    .bubble_b_4a	(bubble_b_4a),
		    .stall_1a		(stall_1a),
		    .stall_2a		(stall_2a),
		    .stall_3a		(stall_3a),
		    .stall_4a		(stall_4a));
   // synthesis translate_on

   // Halt state
   always @(posedge clk or negedge rst_b) begin
      if (!rst_b)
	halted <= 1'b0;
      else
	halted <= exception_halt || syscall_halt;
   end

endmodule // mips_core

