module mips_core_ex(/*AUTOARG*/
   // Outputs
   alu__out_3a, bubble_b_3a, ctrl_RI_3a, ctrl_Sys_3a, ctrl_rddata_3a,
   inst_excpt_3a, mc__size_3a, mc__se_3a, mc__we_3a, mul__rd_data_3a,
   next_alu__out_2a, oflow_halt_3a, pc_3a, reg__rd_num_3a,
   reg__rd_we_3a, reg__rt_data_3a, stall_2a,
   // Inputs
   clk, rst_b, alu__opsel_2a, bubble_b_2a, ctrl_RI_2a, ctrl_Sys_2a,
   ctrl_alurtsel_2a, ctrl_immse_2a, ctrl_rddata_2a, inst_2a,
   inst_excpt_2a, mc__size_2a, mc__se_2a, mc__we_2a, mul__active_2a,
   mul__opcode_2a, pc_2a, reg__rd_num_2a, reg__rd_we_2a,
   reg__rs_data_2a, reg__rt_data_2a, stall_3a, stall_4a
   );

   // Static interface
   output reg [31:0] alu__out_3a;
   output reg 	     bubble_b_3a;
   output reg 	     ctrl_RI_3a;
   output reg 	     ctrl_Sys_3a;
   output reg [1:0]  ctrl_rddata_3a;
   output reg 	     inst_excpt_3a;
   output reg [1:0]  mc__size_3a;
   output reg 	     mc__se_3a;
   output reg 	     mc__we_3a;
   output [31:0]     mul__rd_data_3a;
   output [31:0]     next_alu__out_2a;
   output reg 	     oflow_halt_3a;
   output reg [31:0] pc_3a;
   output reg [4:0]  reg__rd_num_3a;
   output reg 	     reg__rd_we_3a;
   output reg [31:0] reg__rt_data_3a;
   output	     stall_2a;
   input 	     clk, rst_b;
   input [4:0]	     alu__opsel_2a;
   input 	     bubble_b_2a;
   input 	     ctrl_RI_2a;
   input 	     ctrl_Sys_2a;
   input 	     ctrl_alurtsel_2a;
   input 	     ctrl_immse_2a;
   input [1:0]	     ctrl_rddata_2a;
   input [31:0]      inst_2a;
   input 	     inst_excpt_2a;
   input [1:0] 	     mc__size_2a;
   input 	     mc__se_2a;
   input 	     mc__we_2a;
   input 	     mul__active_2a;
   input [2:0] 	     mul__opcode_2a;
   input [31:0]      pc_2a;
   input [4:0] 	     reg__rd_num_2a;
   input 	     reg__rd_we_2a;
   input [31:0]      reg__rs_data_2a;
   input [31:0]      reg__rt_data_2a;
   input 	     stall_3a;
   input 	     stall_4a;
   
   // Internal signals
   wire 	     next_oflow_halt_2a;

   // Auto-wire
   /*AUTOWIRE*/
   // Beginning of automatic wires (for undeclared instantiated-module outputs)
   wire			dcd_bczft;		// From Parser of mips_parse.v
   wire [19:0]		dcd_code;		// From Parser of mips_parse.v
   wire [31:0]		dcd_e_imm;		// From Parser of mips_parse.v
   wire [4:0]		dcd_funct1;		// From Parser of mips_parse.v
   wire [5:0]		dcd_funct2;		// From Parser of mips_parse.v
   wire [15:0]		dcd_imm;		// From Parser of mips_parse.v
   wire [15:0]		dcd_offset;		// From Parser of mips_parse.v
   wire [5:0]		dcd_op;			// From Parser of mips_parse.v
   wire [4:0]		dcd_rd;			// From Parser of mips_parse.v
   wire [4:0]		dcd_rs;			// From Parser of mips_parse.v
   wire [4:0]		dcd_rt;			// From Parser of mips_parse.v
   wire [31:0]		dcd_se_imm;		// From Parser of mips_parse.v
   wire [31:0]		dcd_se_mem_offset;	// From Parser of mips_parse.v
   wire [31:0]		dcd_se_offset;		// From Parser of mips_parse.v
   wire [4:0]		dcd_shamt;		// From Parser of mips_parse.v
   wire [25:0]		dcd_target;		// From Parser of mips_parse.v
   // End of automatics

   // Instruction parsing
   mips_parse Parser(.inst(inst_2a),
		     /*AUTOINST*/
		     // Outputs
		     .dcd_se_imm	(dcd_se_imm[31:0]),
		     .dcd_se_offset	(dcd_se_offset[31:0]),
		     .dcd_e_imm		(dcd_e_imm[31:0]),
		     .dcd_se_mem_offset	(dcd_se_mem_offset[31:0]),
		     .dcd_op		(dcd_op[5:0]),
		     .dcd_funct2	(dcd_funct2[5:0]),
		     .dcd_rs		(dcd_rs[4:0]),
		     .dcd_funct1	(dcd_funct1[4:0]),
		     .dcd_rt		(dcd_rt[4:0]),
		     .dcd_rd		(dcd_rd[4:0]),
		     .dcd_shamt		(dcd_shamt[4:0]),
		     .dcd_offset	(dcd_offset[15:0]),
		     .dcd_imm		(dcd_imm[15:0]),
		     .dcd_target	(dcd_target[25:0]),
		     .dcd_code		(dcd_code[19:0]),
		     .dcd_bczft		(dcd_bczft));
   
   // ALU
   wire [31:0] 		immediate;
   wire [31:0] 		alu__op2;
   assign immediate = ctrl_immse_2a ? dcd_se_imm : dcd_e_imm;
   assign alu__op2 = ctrl_alurtsel_2a ? reg__rt_data_2a : immediate;
   
   mips_alu ALU(.alu__out(next_alu__out_2a),
		.alu__op1(reg__rs_data_2a),
		.alu__shamt(dcd_shamt),
		.alu__opsel(alu__opsel_2a),
		.oflow_halt(next_oflow_halt_2a),
		/*AUTOINST*/
		// Inputs
		.alu__op2		(alu__op2[31:0]));   

   // Multiplier Coprocessor
   multiply_coprocessor MCP(.mul__opcode_2a(mul__opcode_2a),
			    .mul__stall_2a(stall_2a),
			    .rs_data_2a(reg__rs_data_2a),
			    .rt_data_2a(reg__rt_data_2a),
			    /*AUTOINST*/
			    // Outputs
			    .mul__rd_data_3a	(mul__rd_data_3a[31:0]),
			    // Inputs
			    .clk		(clk),
			    .rst_b		(rst_b),
			    .mul__active_2a	(mul__active_2a));

   // Registers
   always @(posedge clk or negedge rst_b) begin
      if (!rst_b) begin
	 /*AUTORESET*/
	 // Beginning of autoreset for uninitialized flops
	 alu__out_3a <= 32'h0;
	 bubble_b_3a <= 1'h0;
	 ctrl_RI_3a <= 1'h0;
	 ctrl_Sys_3a <= 1'h0;
	 ctrl_rddata_3a <= 2'h0;
	 inst_excpt_3a <= 1'h0;
	 mc__se_3a <= 1'h0;
	 mc__size_3a <= 2'h0;
	 mc__we_3a <= 1'h0;
	 oflow_halt_3a <= 1'h0;
	 pc_3a <= 32'h0;
	 reg__rd_num_3a <= 5'h0;
	 reg__rd_we_3a <= 1'h0;
	 reg__rt_data_3a <= 32'h0;
	 // End of automatics
      end else begin
	 if (!stall_3a && !stall_4a) begin
	    // Used
	    alu__out_3a <= next_alu__out_2a;
	    oflow_halt_3a <= next_oflow_halt_2a;
	    
	    // Pass through
	    bubble_b_3a <= bubble_b_2a;
	    ctrl_RI_3a <= ctrl_RI_2a;
	    ctrl_Sys_3a <= ctrl_Sys_2a;
	    ctrl_rddata_3a <= ctrl_rddata_2a;
	    inst_excpt_3a <= inst_excpt_2a;
	    mc__size_3a <= mc__size_2a;
	    mc__se_3a <= mc__se_2a;
	    mc__we_3a <= mc__we_2a;
	    pc_3a <= pc_2a;
	    reg__rd_num_3a <= reg__rd_num_2a;
	    reg__rd_we_3a <= reg__rd_we_2a;
	    reg__rt_data_3a <= reg__rt_data_2a;

	    // We stall
	    if (stall_2a) begin
	       bubble_b_3a <= 1'b0;
	    end

	    // We are a bubble
	    if (!bubble_b_2a) begin
	       ctrl_RI_3a <= 1'b0;
	       ctrl_Sys_3a <= 1'b0;
	       inst_excpt_3a <= 1'b0;
	       mc__we_3a <= 1'b0;
	       oflow_halt_3a <= 1'b0;
	       reg__rd_we_3a <= 1'b0;
	    end
	 end
      end
   end
endmodule // mips_core_ex

