`include "mips_defines.vh"
`include "internal_defines.vh"

module mips_core_mem(/*AUTOARG*/
   // Outputs
   mem_addr, mem_data_in, mem_write_en, bubble_b_4a, ctrl_RI_4a,
   ctrl_Sys_4a, inst_excpt_4a, mc__AdEL_data_4a, mc__AdES_4a,
   mc__DBE_4a, next_reg__rd_data_3a, oflow_halt_4a, pc_4a,
   reg__rd_data_4a, reg__rd_num_4a, reg__rd_we_4a, stall_3a,
   // Inputs
   mem_data_out, mem_excpt, clk, rst_b, alu__out_3a, bubble_b_3a,
   ctrl_RI_3a, ctrl_Sys_3a, ctrl_rddata_3a, inst_excpt_3a,
   mc__size_3a, mc__se_3a, mc__we_3a, mul__rd_data_3a, oflow_halt_3a,
   pc_3a, reg__rd_num_3a, reg__rd_we_3a, reg__rt_data_3a, stall_4a
   );

   // Static interface
   output [29:0]     mem_addr;
   output [31:0]     mem_data_in;
   output [3:0]      mem_write_en;
   output reg 	     bubble_b_4a;
   output reg 	     ctrl_RI_4a;
   output reg 	     ctrl_Sys_4a;
   output reg 	     inst_excpt_4a;
   output reg 	     mc__AdEL_data_4a;
   output reg 	     mc__AdES_4a;
   output reg 	     mc__DBE_4a;
   output reg [31:0] next_reg__rd_data_3a;
   output reg 	     oflow_halt_4a;
   output reg [31:0] pc_4a;
   output reg [31:0] reg__rd_data_4a;
   output reg [4:0]  reg__rd_num_4a;
   output reg 	     reg__rd_we_4a;
   output 	     stall_3a;
   input [31:0]      mem_data_out;
   input 	     mem_excpt;
   input 	     clk, rst_b;
   input [31:0]      alu__out_3a;
   input 	     bubble_b_3a;
   input 	     ctrl_RI_3a;
   input 	     ctrl_Sys_3a;
   input [1:0]	     ctrl_rddata_3a;
   input 	     inst_excpt_3a;
   input [1:0] 	     mc__size_3a;
   input 	     mc__se_3a;
   input 	     mc__we_3a;
   input [31:0]	     mul__rd_data_3a;
   input 	     oflow_halt_3a;
   input [31:0]	     pc_3a;
   input [4:0]	     reg__rd_num_3a;
   input 	     reg__rd_we_3a;
   input [31:0]      reg__rt_data_3a;
   input 	     stall_4a;

   // Internal signals
   wire 	     next_mc__AdEL_data_3a;
   wire 	     next_mc__AdES_3a;
   wire 	     next_mc__DBE_3a;
   wire [31:0] 	     next_mc__out_3a;
   
   // Memory
   memory_controller MemControl(.mc__AdEL_data(next_mc__AdEL_data_3a),
				.mc__AdES(next_mc__AdES_3a),
				.mc__DBE(next_mc__DBE_3a),
				.mc__out(next_mc__out_3a),
				.mc__se(mc__se_3a),
				.mc__we(mc__we_3a),
				.mc__size(mc__size_3a),
				.mc__address(alu__out_3a),
				.mc__in(reg__rt_data_3a),
				/*AUTOINST*/
				// Outputs
				.mem_data_in	(mem_data_in[31:0]),
				.mem_addr	(mem_addr[29:0]),
				.mem_write_en	(mem_write_en[3:0]),
				// Inputs
				.mem_data_out	(mem_data_out[31:0]),
				.mem_excpt	(mem_excpt));

   // Output mux
   always @(*) begin
      next_reg__rd_data_3a = 32'bx;
      case (ctrl_rddata_3a)
	`RDDATA_MC:
	  next_reg__rd_data_3a <= next_mc__out_3a;
	`RDDATA_ALU:
	  next_reg__rd_data_3a <= alu__out_3a;
	`RDDATA_PC:
	  next_reg__rd_data_3a <= pc_3a + 4; // FIXME: Hack to compensate for lack of delay slot
	`RDDATA_MULT:
	  next_reg__rd_data_3a <= mul__rd_data_3a;
      endcase // case (ctrl_rddata)
   end
   
   // Stall condition
   assign stall_3a = 1'b0;
   
   // Registers
   always @(posedge clk or negedge rst_b) begin
      if (!rst_b) begin
	 /*AUTORESET*/
	 // Beginning of autoreset for uninitialized flops
	 bubble_b_4a <= 1'h0;
	 ctrl_RI_4a <= 1'h0;
	 ctrl_Sys_4a <= 1'h0;
	 inst_excpt_4a <= 1'h0;
	 mc__AdEL_data_4a <= 1'h0;
	 mc__AdES_4a <= 1'h0;
	 mc__DBE_4a <= 1'h0;
	 oflow_halt_4a <= 1'h0;
	 pc_4a <= 32'h0;
	 reg__rd_data_4a <= 32'h0;
	 reg__rd_num_4a <= 5'h0;
	 reg__rd_we_4a <= 1'h0;
	 // End of automatics
      end else begin
	 if (!stall_4a) begin
	    // Used
	    mc__AdEL_data_4a <= next_mc__AdEL_data_3a;
	    mc__AdES_4a <= next_mc__AdES_3a;
	    mc__DBE_4a <= next_mc__DBE_3a;
	    reg__rd_data_4a <= next_reg__rd_data_3a;
	    
	    // Pass through
	    bubble_b_4a <= bubble_b_3a;
	    ctrl_RI_4a <= ctrl_RI_3a;
	    ctrl_Sys_4a <= ctrl_Sys_3a;
	    inst_excpt_4a <= inst_excpt_3a;
	    oflow_halt_4a <= oflow_halt_3a;
	    pc_4a <= pc_3a;
	    reg__rd_num_4a <= reg__rd_num_3a;
	    reg__rd_we_4a <= reg__rd_we_3a;
	    
	    // We are a bubble
	    if (!bubble_b_3a) begin
	       ctrl_RI_4a <= 1'b0;
	       ctrl_Sys_4a <= 1'b0;
	       inst_excpt_4a <= 1'b0;
	       mc__AdEL_data_4a <= 1'b0;
	       mc__AdES_4a <= 1'b0;
	       mc__DBE_4a <= 1'b0;
	       oflow_halt_4a <= 1'b0;
	       reg__rd_we_4a <= 1'b0;
	    end
	 end
      end
   end
endmodule // mips_core_if
