module mips_register_file(/*AUTOARG*/
   // Outputs
   reg__rs_data, reg__rt_data, stall_1a,
   // Inputs
   clk, halted, rst_b, mc__size_2a, mc__we_2a, mul__active_2a,
   next_alu__out_2a, next_reg__rd_data_3a, next_reg__rd_num_1a,
   next_reg__rd_we_1a, reg__rs_num, reg__rs_re, reg__rt_num,
   reg__rt_re, reg__rd_num, reg__rd_data, reg__rd_we, stall_2a,
   stall_3a, stall_4a
   );

   // Static interface
   output reg [31:0] reg__rs_data;
   output reg [31:0] reg__rt_data;
   output	     stall_1a;
   input 	 clk, halted, rst_b;
   input [1:0]	 mc__size_2a;
   input 	 mc__we_2a;
   input 	 mul__active_2a;
   input [31:0]  next_alu__out_2a;
   input [31:0]  next_reg__rd_data_3a;
   input [4:0] 	 next_reg__rd_num_1a;
   input  	 next_reg__rd_we_1a;
   input [4:0] 	 reg__rs_num;
   input 	 reg__rs_re;
   input [4:0] 	 reg__rt_num;
   input 	 reg__rt_re;
   input [4:0] 	 reg__rd_num;
   input [31:0]  reg__rd_data;
   input 	 reg__rd_we;
   input 	 stall_2a;
   input 	 stall_3a;
   input 	 stall_4a;

   // Auto wire
   /*AUTOWIRE*/
   // Beginning of automatic wires (for undeclared instantiated-module outputs)
   wire [1:0]		queue__rs_where;	// From RWQ of reg_write_queue.v
   wire [1:0]		queue__rt_where;	// From RWQ of reg_write_queue.v
   wire [31:0]		rs_data;		// From BS of regfile_3port.v
   wire [31:0]		rt_data;		// From BS of regfile_3port.v
   // End of automatics
   
   // Backing store
   regfile_3port BS(.rs_num(reg__rs_num),
		    .rt_num(reg__rt_num),
		    .rd_num(reg__rd_num),
		    .rd_data(reg__rd_data),
		    .rd_we(reg__rd_we),
		    /*AUTOINST*/
		    // Outputs
		    .rs_data		(rs_data[31:0]),
		    .rt_data		(rt_data[31:0]),
		    // Inputs
		    .clk		(clk),
		    .rst_b		(rst_b),
		    .halted		(halted));

   // Hazard detection
   reg_write_queue RWQ(.queue__rd_num(next_reg__rd_num_1a),
		       .queue__rd_we(stall_1a ? 1'b0 : next_reg__rd_we_1a),
		       .queue__rs_num(reg__rs_num),
		       .queue__rs_re(reg__rs_re),
		       .queue__rt_num(reg__rt_num),
		       .queue__rt_re(reg__rt_re),
		       .queue__enable(!stall_2a && !stall_3a && !stall_4a), // FIXME: Hack
		       /*AUTOINST*/
		       // Outputs
		       .queue__rs_where	(queue__rs_where[1:0]),
		       .queue__rt_where	(queue__rt_where[1:0]),
		       // Inputs
		       .clk		(clk),
		       .rst_b		(rst_b));

   // Stall logic
   wire 		load_stall;
   wire 		mcp_stall;

   // We can't forward out of the ALU if there's a pending memory read
   assign load_stall = mc__size_2a != 2'b0 &&
		       mc__we_2a == 1'b0 &&
		       ((queue__rs_where == 2'd3 && reg__rs_re) ||
			(queue__rt_where == 2'd3 && reg__rt_re)) ? 1'b1 : 1'b0;

   // We also can't forward out of the ALU if the result is coming out of the MCP
   assign mcp_stall = mul__active_2a != 2'b0 &&
		      ((queue__rs_where == 2'd3 && reg__rs_re) ||
		       (queue__rt_where == 2'd3 && reg__rt_re)) ? 1'b1 : 1'b0;

   assign stall_1a = load_stall || mcp_stall;
   
   // Forwarding logic
   always @(*) begin
      reg__rs_data <= 'bx;
      reg__rt_data <= 'bx;
      
      if (reg__rs_re)
	case (queue__rs_where)
	  2'd0:
	    reg__rs_data <= rs_data;
	  2'd1:
	    reg__rs_data <= reg__rd_data;
	  2'd2:
	    reg__rs_data <= next_reg__rd_data_3a;
	  2'd3:
	    reg__rs_data <= next_alu__out_2a;
	endcase // case (queue__rs_where)

      if (reg__rt_re)
	case (queue__rt_where)
	  2'd0:
	    reg__rt_data <= rt_data;
	  2'd1:
	    reg__rt_data <= reg__rd_data;
	  2'd2:
	    reg__rt_data <= next_reg__rd_data_3a;
	  2'd3:
	    reg__rt_data <= next_alu__out_2a;
	endcase // case (queue__rt_where)
   end
endmodule // mips_register_file

