module reg_write_queue(/*AUTOARG*/
   // Outputs
   queue__rs_where, queue__rt_where,
   // Inputs
   clk, rst_b, queue__rd_num, queue__rd_we, queue__rs_num,
   queue__rs_re, queue__rt_num, queue__rt_re, queue__enable
   );

   parameter N = 3;
   
   output reg [1:0] queue__rs_where;
   output reg [1:0] queue__rt_where;
   input 	    clk, rst_b;
   input [4:0] 	    queue__rd_num;
   input 	    queue__rd_we;
   input [4:0] 	    queue__rs_num;
   input 	    queue__rs_re;
   input [4:0] 	    queue__rt_num;
   input 	    queue__rt_re;
   input 	    queue__enable;
   
   reg [4:0]   queue [N-1:0];
   reg [1:0]   i;

   // Combinational read
   always @(*) begin
      queue__rs_where <= 2'b0;
      queue__rt_where <= 2'b0;
      if (queue__enable) begin
	 for (i = 0; i < N; i = i + 1) begin
	    // Is a write pending?
	    if (queue__rs_num != 0 && queue__rs_re && queue__rs_num == queue[i])
	       queue__rs_where <= i + 1;
	    if (queue__rt_num != 0 && queue__rt_re && queue__rt_num == queue[i])
	       queue__rt_where <= i + 1;
	 end
      end
   end

   // Sequential update
   always @(posedge clk or negedge rst_b) begin
      if (!rst_b) begin
	 for (i = 0; i < N; i = i + 1) begin
	    queue[i] <= 0;
	 end
      end else begin
	 if (queue__enable) begin
	    // Index queue
	    for (i = 0; i < N; i = i + 1) begin
	       if (i != N - 1)
		 queue[i] <= queue[i + 1];
	       else
		 queue[i] <= queue__rd_we ? queue__rd_num : 5'b0;
	    end
	 end
      end
   end
   
endmodule